/************************************************************/
/* The CRAY LOOP Test - SMART COMMUNICATIONS, Inc. New York */
/* Written in ANSI-C for the IBM RS/6000 - loop test        */
/* Author: Lee Atchison - Programming Staff - SMARTCOM      */
/* Purpose to get an idea of how much faster a CRAY runs    */
/************************************************************/
#include <stdio.h>   /* declare standard Unix headers */
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
/* #include <sys\timeb.h>*/

/* Test a define on the compiler */
#define MEMCLR(X)   memset(X,'\0',sizeof(X))
#define OURLOOPS 20000
/* Increase this define for a wider sampling - 1000 */

int main()
{
FILE *fp1;           /* this will test opening and closing a file */
char sentence[256];  /* An ordinary array to hold a sentence */
char *aptr;          /* a pointer to reference our memory sentence */
char myword[80];     /* local word holder */
int row,j,k,posflag,wcount;  /* int's - but can be shorts - this is wasteful */
/************ TIMING HERE ***********/
time_t stime,etime;  /* start of the loop time */
time_t lng_time;
/* This will be adjusted on the Cray because it will take fractions of second */
stime = time( &lng_time ); /* get the start time in seconds */
/**************** End of Timing **********/

/* The Cray may give us a null fill on declare - RS6000 does not */
/* Non - macro form */
memset(sentence,'\0',sizeof(sentence));  /* waste a few cycles here */

/* Now pretend we got text from the message handler */
strcpy(sentence,"Now is the time for all good men to come to the aid of their country. The walrus: has big tusks. The igloos, loom in the dusk.@@");
/* the double at signs are the master message terminators */

if ((fp1 = fopen("SMART.DOC","w"))==NULL)
    { /* reality check - Cray should not fail here */
    printf("Can not open SMART.DOC\n");
    exit(1);
    }
/* now point at the sentence and do a primary parse */
/* simulate say 3000 sentences */

for (j=0; j<OURLOOPS; j++)
    {
    MEMCLR(myword);  /* waste more cycles */
    row=0;  /* test an integer counter */
    k=0;    /* local counter for word  */
    posflag=0; /* never used */
    wcount=0;  /* count the words */

for (aptr=sentence; *aptr != '\0'; aptr++,row++)
    { /* This loop will rupture is the Cray misses the loop */
      /* Possible stress test is to load the sentence to 255 bytes */
      /* Other causes may be NO SPACES in sentence */
    switch(*aptr)
        {
    case '@': /* stopper is two */
            if (*(aptr+1)=='@') /* stop the flow */
               {
            if (k>0)
               {  /* STORES A WORD IN A LIST */
               if (add_tword(myword,row-k,posflag,k,wcount,fp1) < 0) return(-1);
               MEMCLR(myword);
               k=0; wcount++; 
               }
               }
            else myword[k++] = *aptr;
            break;
    case ' ':  /* common word terminator */
            if (k>0)
               {
               if (add_tword(myword,row-k,posflag,k,wcount,fp1) < 0) return(-1);
               MEMCLR(myword);
               k=0; wcount++; 
               }
             break;
     case '.': case ',': case ':':
            if (k>0)
               {
               if (add_tword(myword,row-k,posflag,k,wcount,fp1) < 0) return(-1);
               MEMCLR(myword);
               k=0; wcount++; 
               }
             myword[k++] = *aptr;
             break;
    default: myword[k++] = *aptr; /* remove spaces and Cray will crash here */
             break;
       }
    }
    } /* simulate 100 sentences */
/***** TIME STOPS HERE *****************/
etime = (time( &lng_time ) - stime);
fprintf(fp1,"\nThe Cray time is %ld seconds\n",etime);
fclose(fp1); /* close the file */
}

int add_tword(myword,row,posflag,k,wcount,fp1)
char *myword;
int row,posflag,k,wcount;
FILE *fp1;
{
/* What happens if variables are passed but not used - compiler test */
/* test a write to file - this will slow the Cray */
/* fprintf(fp1,"WORD No. %d is %s for %d chars\n",wcount,myword,k); */
return(1);  /* return is always true in the test */
}
